<?php

#
#   FILE:  SPT--ExportDataExecute.php
#
#   FUNCTIONS PROVIDED:
#       ExportComplete()
#       PrintResourceCount()
#       PrintExportLink()
#
#   FUNCTIONS EXPECTED:
#       None
#
#   Part of the Scout Portal Toolkit
#   Copyright 2003 Internet Scout Project
#   http://scout.cs.wisc.edu
#

require_once("include/SPT--Common.php");
require_once("include/SPT--MetadataSchema.php");
require_once("include/SPT--SPTDate.php");

# ----- EXPORTED FUNCTIONS ---------------------------------------------------

# see if export is complete
function ExportComplete()
{
    global $Session;

    $ExportComplete = $Session->Get("ExportComplete");

    return ($ExportComplete == 1) ? true : false;
}

# print Resource Count
function PrintResourceCount()
{
    global $Session;

    print $Session->Get("ResourceCount");
}

# print link to downloadable exported file
function PrintExportLink()
{
    global $Session, $NavDirCorrection;

    print "<a href=\"".$NavDirCorrection.$Session->Get("FileName")."\">";
}

# ----- LOCAL FUNCTIONS ------------------------------------------------------

# init variables -- first time thru they will be null
function InitExportVars()
{
    global $ExportComplete, $FSeek, $ResourceCount, $TotalLineCount;
    global $ControlledNameCount, $ClassificationCount, $InputLineCount;

    if(is_null($ExportComplete))
        $ExportComplete = 0;
    if(is_null($FSeek))
        $FSeek = 0;
    if(is_null($ResourceCount))
        $ResourceCount = 0;
    if(is_null($InputLineCount))
        $InputLineCount = 0;
    if(is_null($TotalLineCount))
        $TotalLineCount = 0;
    if(is_null($ControlledNameCount))
        $ControlledNameCount = 0;
    if(is_null($ClassificationCount))
        $ClassificationCount = 0;
}

# do/while loop, this is the main event
function DoWhileLoop()
{
    global $DB, $DB1, $DB2, $fpi, $fpo, $Session, $ExportComplete;
    global $User, $ResourceCount, $FSeek, $TotalLineCount;

    # build partial query for dynamic fields
    $Schema = & new MetadataSchema();
    $Fields = $Schema->GetFields(MDFTYPE_TEXT | MDFTYPE_PARAGRAPH | 
                MDFTYPE_NUMBER | MDFTYPE_DATE | MDFTYPE_TIMESTAMP | 
                MDFTYPE_FLAG | MDFTYPE_USER);
            
    # loop through each field
    foreach($Fields as $Field)
    {
        if($Field->Enabled())
        {
            $FName = $Field->Name();
            $DBName = $Field->DBFieldName();
            if($Field->Type() & MDFTYPE_DATE)
            {
                $Dynamic .= $DBName."Begin, ";
                $Dynamic .= $DBName."End, ";
                $Dynamic .= $DBName."Precision, ";
                $Header .= $FName."\t";
            }
            else
            {
                $Dynamic .= $DBName.", ";
                $Header .= $FName."\t";
            }
        }
    }

    $Header .= "ControlledName\t".
                    "ControlledNameTypeName\t".
                    "ClassificationName\t".
                    "ClassificationTypeId\n";


    # write out the header line only on first time
    if(!empty($fpo) && $FSeek == 0)
        fwrite($fpo, $Header);
#    else
#        print $Header;
        
    $InputLineCount = 0;

    # strip off trailing comma
    $Dynamic = substr($Dynamic, 0, strlen($Dynamic) - 2);
    $Dynamic .= " ";

    while(!feof($fpi) && $InputLineCount < 50 && $ExportComplete == 0)
    {
        $ResourceId = fgets($fpi, 4096);
        $FSeek += strlen($ResourceId);
        $ResourceId = trim($ResourceId);

        if(empty($ResourceId))
            break;
        
        $ResourceCount++;
        $InputLineCount++;

        $query = "SELECT distinct ".$Dynamic.
            " from Resources where ResourceId =".  $ResourceId;

#        echo "query=$query<br>";
        $DB->Query($query);

#    $Fields = $Schema->GetFields();
        while($Result = $DB->FetchNextRowArray())
        {
            $Record = NULL;
            
            # loop through each field
            foreach($Fields as $Field)
            {
                if($Field->Enabled())
                {
                    # declare global form variables
                    $FName = $Field->DBFieldName();
                    if($Field->Type() & MDFTYPE_DATE)
                    {
                        $Date = & new SPTDate($Result[$FName."Begin"],
                            $Result[$FName."End"], $Result[$FName."Precision"]);
                        $Record .= $Date->Formatted()."\t";
                    }
                    else
                    {
                        # strip out linefeeds, carriage returns, and tabs
                        $DataValue = $Result[$FName];
                        $DataValue = preg_replace("/([\t\r\n])/", " ", $DataValue);
                        $Record .= $DataValue."\t";
                    }
                }
            }

            # process controlled names and classifications next
            $query = "SELECT distinct ControlledName, ControlledNameTypes.ControlledNameTypeName from ControlledNames, ResourceNameInts, ResourceClassInts, ControlledNameTypes where ResourceNameInts.ResourceId = $ResourceId and ResourceNameInts.ControlledNameId=ControlledNames.ControlledNameId and ControlledNames.ControlledNameTypeId=ControlledNameTypes.ControlledNameTypeId";
            $DB1->Query($query);

            $query = "SELECT distinct ClassificationName, ClassificationTypes.ClassificationTypeId from Classifications, ResourceClassInts, ClassificationTypes where ResourceClassInts.ResourceId = $ResourceId and ResourceClassInts.ClassificationId=Classifications.ClassificationId and Classifications.ClassificationTypeId = ClassificationTypes.ClassificationTypeId";
            $DB2->Query($query);

            # no controlled names or classificatons so output what we have
            if($DB1->NumRowsSelected() == 0 && $DB2->NumRowsSelected() == 0)
            {
                $Record .= "\t\t\t\n";

                if(!empty($fpo))
                    fwrite($fpo, $Record);
                else
                    print $Record;
                $TotalLineCount++;
                continue;
            }

            while($Result = $DB1->FetchNextRowArray())
            {
                # if no ControlledName, set ControlledNameTypeName to NULL;
                if(empty($Result[ControlledName]))
                    $Result[ControlledNameTypeName] = NULL;

                $OutputLine = $Record;
                $OutputLine .= $Result["ControlledName"]."\t";
                $OutputLine .= $Result["ControlledNameTypeName"]."\t\t\n";

#                echo "Output=$OutputLine<br>";

                if(!empty($fpo))
                    fwrite($fpo, $OutputLine);
                else
                    print $OutputLine;
                $TotalLineCount++;
            }

            while($Result = $DB2->FetchNextRowArray())
            {
                # if no Classification, set ClassificationTypeId to NULL
                if(empty($Result[ClassificationName]))
                    $Result[ClassificationName] = NULL;
 
                $OutputLine = $Record."\t\t";
                $OutputLine .= $Result["ClassificationName"]."\t";
                $OutputLine .= $Result["ClassificationTypeId"]."\n";

#                echo "Output=$OutputLine<br>";

                if(!empty($fpo))
                    fwrite($fpo, $OutputLine);
                else
                    print $OutputLine;
                $TotalLineCount++;
            }
        }
    }
}

# function to get list of matching files
function ListDir($DirPath, $Pattern)
{
    static $ResultArray = array();  

    $Handle = opendir($DirPath);
    while($File = readdir($Handle))
    {
        if($File == '.' || $File == '..')
            continue;
        if(is_dir($DirPath.$File))
            continue;
        else if(preg_match($Pattern, $File))
            $ResultArray[]=$DirPath.$File;
    }
    closedir($Handle);
    return $ResultArray;
}

# ----- MAIN -----------------------------------------------------------------

$NavDirCorrection = "../";

# check if current user is authorized
CheckAuthorization(PRIV_SYSADMIN);

$DB =& new SPTDatabase();
$DB1 =& new SPTDatabase();
$DB2 =& new SPTDatabase();
$ClassDB =& new SPTDatabase();

# open import file for reading
if(is_null($InputPath))
{
    $ScriptFileName = $HTTP_SERVER_VARS['SCRIPT_FILENAME'];

    $InputPath = substr($ScriptFileName, 0, strrpos($ScriptFileName, '/')).
        "/".$NavDirCorrection."TempStorage/InputData";
}

if($Submit == "Cancel")
{
    PrintAutoRefreshPage("System Administration",$NavDirCorrection.
        "SPT--SysAdmin.php");
    exit;
}

# export all records, create new InputData file
if($Submit == "Export All Resources")
{
    $Query = "Select ResourceId from Resources order by ResourceId";
    $DB->Query($Query);

    $fp = fopen($InputPath, "w");

    while($Result = $DB->FetchNextRowArray())
    {
       fwrite($fp, $Result[ResourceId]."\n");
    }
    fclose($fp);
}

InitExportVars();

# open input file for reading
if(is_readable($InputPath))
{
    $fpi = fopen($InputPath, "r");
}
else
{
    $ErrorMessage = "Cannot open Resource ID Filename: $InputPath<br>";
    $Session->PassVariable("ErrorMessage", $ErrorMessage);
    PrintAutoRefreshPage("ErrorMessage", "SPT--DisplayError.php");
    exit;
}

# open export path
if(is_null($ExportPath))
{
    $ScriptFileName = $HTTP_SERVER_VARS['SCRIPT_FILENAME'];
    $FileName = "TempStorage/Metadata_".date("YmdHis").".txt";
    $ExportPath = dirname($ScriptFileName)."/".$NavDirCorrection.$FileName;

    # remove any old exported files
    $OldExportFiles = ListDir(dirname($ScriptFileName)."/".$NavDirCorrection.
        "TempStorage/", "/^Metadata_.*\.txt$/");

    if(is_array($OldExportFiles))
    {
        foreach($OldExportFiles as $OldFile)
        {
            unlink($OldFile);
        }
    }
}

$fpo = fopen($ExportPath, "a");
if($fpo == FALSE)
{
    $ErrorMessage = "Cannot open Export Filename: $ExportPath<br>";
    $Session->PassVariable("ErrorMessage", $ErrorMessage);
    PrintAutoRefreshPage("ErrorMessage", "SPT--DisplayError.php");
    exit;
}

# seek to the next line
if($FSeek > 0)
{
    fseek($fpi, $FSeek);
}

# the main work happenes here
DoWhileLoop();

if (feof($fpi))
{
    $ExportComplete = 1;
}

$Session->PassVariable("ExportComplete", $ExportComplete);
$Session->PassVariable("ResourceCount", $ResourceCount);
$Session->PassVariable("FileName", $FileName);
$Session->PassVariable("TotalLineCount", $TotalLineCount);
$Session->PassVariable("FSeek", $FSeek);
$Session->PassVariable("InputPath", $InputPath);
$Session->PassVariable("ExportPath", $ExportPath);

#  Time to auto-refresh?
if($ExportComplete == 0)
{
    $AutoRefreshToPage = "SPT--ExportDataExecute.php";
}

PageTitle("Export Data");
$NavDirCorrection = "../";
include("include/SPT--StandardHtmlPageLoad.php");

if($ExportComplete == 1)
{
    fclose($fpi);
    fclose($fpo);
    unlink($InputPath);
}

?>
